@@ -5,6 +5,7 @@ from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord 
 from Bio.Align import MultipleSeqAlignment 
 import inspect
+import copy
 import sys
 import os
 import re
@@ -69,11 +70,66 @@ def search_features_for_patterns(features, mge_patterns):
   return(mges)
 
 def find_mges(genome_file_path, file_format, mge_file_path = None):
+  """
+  returns mges as a list of dicts with the keys: type, start, end, strand, description
+  """
   features = get_features(genome_file_path, file_format)
   mge_patterns = get_mge_patterns(mge_file_path)
   mges = search_features_for_patterns(features, mge_patterns)
   return(mges)
 
+
+def merge_mges(mges, merge_interval):
+  previous_mge = {}
+  merged_mge = { "type": None, "start": None, "end": None, "strand": None, "description": {}}
+  number_mges_merged = 1
+  merged_mges = []
+  for mge in mges:
+    if not previous_mge:
+      previous_mge = mge
+    else:
+      # check if this mge should be merged with previous
+      if mge['start'] - previous_mge['end'] <= merge_interval: # merge if less than interval
+        if not merged_mge['start']:
+          merged_mge['start'] = previous_mge['start']
+        merged_mge['end'] = mge['end']
+        if not merged_mge['strand']:
+          merged_mge['strand'] = previous_mge['strand']
+        if mge['strand'] != merged_mge['strand']:
+            merged_mge['strand'] = '.'
+
+        if not merged_mge['description']:
+          merged_mge['description'] = copy.deepcopy(previous_mge['description'])
+        # merge the descriptions
+        for match in mge['description']:
+          if match not in merged_mge['description']:
+            merged_mge['description'][match] = []
+          merged_mge['description'][match].extend(mge['description'][match])
+        number_mges_merged += 1
+      else:
+        # check if a mge has been merged
+        if merged_mge['start']:
+          merged_mge['type'] = f"{number_mges_merged} merged features"
+          merged_mges.append(merged_mge)
+          # reset merged_mge
+          merged_mge = { "type": None, "start": None, "end": None, "strand": None, "description": {}}
+          number_mges_merged = 1
+        else:
+          merged_mges.append(previous_mge)
+      previous_mge = mge
+    
+    # Final merge
+  if merged_mge['start']:
+    merged_mge['type'] = f"{number_mges_merged} merged features"
+    merged_mges.append(merged_mge)
+    # reset merged_mge
+    merged_mge = { "type": None, "start": None, "end": None, "strand": None, "description": {}}
+  else:
+    merged_mges.append(previous_mge)
+
+  return(merged_mges)
+        
+
 def create_mge_gff_file(genome_file_path, file_format, mges):
   record = SeqIO.read(genome_file_path, file_format)
   gff_file_path = Path(genome_file_path).with_suffix('.gff')
@@ -109,7 +165,9 @@ def mask_mges(pseudogenome_alignment_path, gff_file_path, masking_character):
   check_alignment(alignment, gff_ranges)
 
   masked_sequences = []
-  for record in alignment:
+  for index, record in enumerate(alignment):
+    if index+1 > 0 and (index+1) % 10 == 0:
+      print(f'Masking sequence {index+1}')
     sequence = record.seq
     for gff_range in gff_ranges:
       start = gff_range[0] -1
